/*============================================================================
  File:     Merge.sql

  Summary:  Simple examples of MERGE statement in SQL Server 2008.
  
			Examining the query plans, the first example (with WHEN MATCHED and
			WHEN NOT MATCHED IN TARGET) uses a LEFT OUTER JOIN in the plan. 
			This appears at the far right in the graphic showplan.
			
			The second example (with WHEN MATCHED, WHEN NOT MATCHED IN TARGET 
			and WHEN NOT MATCHED IN SOURCE) uses the equivalent of a FULL 
			OUTER JOIN (LEFT OUTER JOIN concatenated with LEFT ANTI-SEMI JOIN)
			in the plan.

			Also demonstrates using the OUTPUT clause and it's $action variable
			with MERGE.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE tempdb
go

IF OBJECTPROPERTY(OBJECT_ID('dbo.Customers'), 'IsTable') = 1
  drop table dbo.Customers
GO
IF OBJECTPROPERTY(OBJECT_ID('dbo.QuarterlyUpdate'), 'IsTable') = 1
  drop table dbo.QuarterlyUpdate
GO

--
-- Step 1: Create and Populate Customers table.
--         Note: Population uses row constructors, which are new in SQL Server 2008
--

CREATE TABLE dbo.Customers (
  customer_id			INT IDENTITY PRIMARY KEY,
  customer_name			VARCHAR(30),
  policy_type			INT        CHECK(policy_type > 0 AND policy_type < 4),
  deductable			MONEY,
  number_of_claims      INT
)
go

INSERT Customers VALUES
  ('Robert Smith', 1, 1000, 0),
  ('Mary Johnson', 1, 5000, 0),
  ('Fred Paulson', 2, 2000, 1),
  ('Nancy Jones',  2, 1000, 1)  
GO

--
-- Step 2: Create and Populate QuarterlyUpdate table.
--         Note: Population uses row constructors, which are new in SQL Server 2008
--         Note: Not every Customer must appear in the QuarterlyUpdate table.
--
  
CREATE TABLE QuarterlyUpdate (
   customer_id			INT,
   customer_name		VARCHAR(30),
   policy_type			INT        CHECK(policy_type > 0 AND policy_type < 4),
   deductable			MONEY,
   new_claims           INT
 )
 GO
 
 INSERT QuarterlyUpdate VALUES
  (1, 'Robert Jones', 1, 1000, 0),
  (2, 'Mary Johnson', 1, 5000, 2),
  (0, 'Katy Fredericks', 2, 2000, 0),   -- New Customer 
  (0, 'Phil Fredericks', 2, 2000, 0)   -- New Customer 
GO

--
-- Step 3: MERGE
--         Note: No action is take when row appears in Customer but not QuerterlyUpdate
--         Note: Uses increment operator, new in SQL Server 2008 T-SQL
--

MERGE Customers C
  USING QuarterlyUpdate Q
  ON C.customer_id = Q.customer_id
  WHEN MATCHED THEN 
    UPDATE SET C.customer_name = Q.customer_name,
               C.policy_type = Q.policy_type,
               C.deductable = Q.deductable,
               C.number_of_claims += new_claims
  WHEN NOT MATCHED THEN
    INSERT VALUES(customer_name, policy_type, deductable, new_claims)
    OUTPUT $action, INSERTED.customer_id, 
					INSERTED.customer_name as [New Name], 
					INSERTED.policy_type as [New Policy], 
					INSERTED.deductable as [New Deductable], 
					INSERTED.number_of_claims as [New Number Of Claims],
					DELETED.customer_name as [Original Name],
					DELETED.policy_type as [Original Policy], 
					DELETED.deductable as [Original Deductable], 
					DELETED.number_of_claims as [Original Number Of Claims];
GO

DELETE QuarterlyUpdate;

--
-- Step 5:  Next quarter, a new business rule is introduced.
--			A row for each current customer is required in QuarterlyUpdate
--			If no row is present, customer will  be deleted
--

INSERT QuarterlyUpdate VALUES
  (1, 'Robert Jones', 1, 1000, 0),
  (2, 'Mary Johnson', 1, 5000, 2),
  (3, 'Fred Paulson', 2, 2000, 1),
  (5, 'Katy Fredericks', 2, 2000, 1),   
  (6, 'Phil Fredericks', 2, 2000, 0),
  (0, 'Sam Fielding', 3, 10000, 0)  
GO

--
-- Step 3: MERGE
--         Note: When row appears in Customer but not QuerterlyUpdate, Customer is deleted
--         Note: Uses increment operator, new in SQL Server 2008 T-SQL
--         Note: Uses WHEN NOT MATCHED BY SOURCE  - keyword SOURCE is required
--          and  Uses WHEN NOT MATCHED BY TARGET  - keyword TARGET is optional
--

MERGE Customers C
  USING QuarterlyUpdate Q
  ON C.customer_id = Q.customer_id
  WHEN MATCHED THEN 
    UPDATE SET C.customer_name = Q.customer_name,
               C.policy_type = Q.policy_type,
               C.deductable = Q.deductable,
               C.number_of_claims += new_claims
  WHEN NOT MATCHED BY SOURCE THEN
     DELETE
  WHEN NOT MATCHED BY TARGET THEN
    INSERT VALUES(customer_name, policy_type, deductable, new_claims)
    OUTPUT $action, INSERTED.customer_id, 
					INSERTED.customer_name as [New Name], 
					INSERTED.policy_type as [New Policy], 
					INSERTED.deductable as [New Deductable], 
					INSERTED.number_of_claims as [New Number Of Claims],
					DELETED.customer_name as [Original Name],
					DELETED.policy_type as [Original Policy], 
					DELETED.deductable as [Original Deductable], 
					DELETED.number_of_claims as [Original Number Of Claims];
GO